% Markus Kowalsky
% The Johns Hopkins University

%$%%%%%%%%%%%%%%%%%    PARAMETERS   %%%%%%%%%%%%%%%%%%$%
%$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%
%$ map_types                                          $%
%$   1) 'INTRACARDIAC'                                $%
%$   2) 'BODY SURFACE'                                $%
%$ plot_type                                          $%
%$   1) 'SAI QRST'                                    $%
%$   2) 'QRST INTEGRAL'                               $%
%$   3) 'P2P VOLTAGE'                                 $%
%$   4) 'TOTAL REPOLARIZATION TIME'                   $%
%$   5) 'ACTIVATION TIME'                             $%
%$   6) 'ACTIVATION RECOVERY INTERVAL'                $%
%$ data (just input data for each lead)               $%
%$ color_type                                         $%
%$   1) 'NORMAL' (red high, blue low)                 $%
%$   2) 'INVERTED' (blue high, red low)               $%
%$ view_type                                          $%
%$   1) 'FRONT' (Body Surface)                        $%
%$   2) 'Back' (Body Surface)                         $%
%$   3) 'ANTERIOR POSTERIOR'                          $%
%$   4) 'LEFT POSTERIOR OBLIQUE'                      $%
%$ *all should be case-insenstive                     $%
%$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%
%$%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%$%

function map_handle = create_map(map_type, plot_type, data, color_type, view_type)
load nodes_b120_h291.mat;
load triangles_b236_h578.mat;

%% Processing inputs
% make parameters case-insensitive
map_type = upper(map_type);
plot_type = upper(plot_type);
color_type = upper(color_type);
disp(['Creating ', map_type, ' map of ', plot_type, ' with ',...
  color_type, ' color scheme...']);

% make sure data is right dimension
data_dim = size(data);
if data_dim(1,1) == 1
  data = data';
end

%% Plotting

% figure attributes
map_handle = figure('Renderer','opengl'); hold on; axis vis3d;
h = colorbar('FontSize',14,'FontName','Gill Sans');
cmap = get(map_handle,'Colormap');
if strcmp(color_type,'INVERTED')
  set(map_handle,'Colormap',flipud(cmap));
end

% plot actual data
if strcmp(map_type,'BODY SURFACE')
  patch('Vertices',nodes.b,'Faces',triangles.b,'FaceVertexCData',data,...
    'FaceColor','interp','EdgeAlpha',0,'EdgeColor','None','FaceAlpha',1);
elseif strcmp(map_type,'INTRACARDIAC')
  patch('Vertices',nodes.h,'Faces',triangles.h,'FaceVertexCData',data,...
    'FaceColor','interp','EdgeAlpha',0,'EdgeColor','None','FaceAlpha',1);
end
axis off; axis equal;
cameratoolbar('SetMode','orbit');
cameratoolbar('SetCoordSys','none');

% Set units on color scale
if strcmp(plot_type,'SAI QRST')
  xlabel(h,'mV*ms');
elseif strcmp(plot_type,'QRST INTEGRAL')
  xlabel(h,'mV*ms');
elseif strcmp(plot_type,'P2P VOLTAGE')
  xlabel(h,'mV');
elseif strcmp(plot_type,'TOTAL REPOLARIZATION TIME')
  xlabel(h,'ms');
elseif strcmp(plot_type,'ACTIVATION TIME')
  xlabel(h,'ms');
elseif strcmp(plot_type,'ACTIVATION RECOVERY INTERVAL')
  xlabel(h,'ms');
end

% Set view
if strcmp(view_type,'FRONT')
  view(-2,26);
elseif strcmp(view_type,'BACK')
 view(192,-8);
elseif strcmp(view_type,'ANTERIOR POSTERIOR')
  view(9,52);
elseif strcmp(view_type,'LEFT POSTERIOR OBLIQUE')
  view(189,-52);
end

% Set Figure Name
if strcmp(map_type,'BODY SURFACE')
  name_string = ['BODY SURFACE MAP OF ', plot_type, '-', view_type];
  set(map_handle,'Name',name_string);
elseif strcmp(map_type,'INTRACARDIAC')
  name_string = [view_type, ' VIEW OF ', plot_type];
  set(map_handle,'Name',name_string);
end

colormapeditor % for editing colormaps
end