% Markus Kowalsky
% The Johns Hopkins University

%$%%%%%%%%%%%%%%%%%%%%%%%%%    PARAMETERS    %%%%%%%%%%%%%%%%%%%%%%%%%$%
%$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%
%$ data (just input data for each lead)                               $%
%$ sampling_rate (Hz)                                                 $%
%$ run_type                                                           $%
%$   1) 'NORMAL'                                                      $%
%$   2) 'DEBUG-A' (show plots of fiducial points on original data)    $%
%$   3) 'DEBUG-B' (show everything)                                   $%
%$ *all should be case-insenstive                                     $%
%$ threshold_value (threshold data must cross for QRS detection algo.)$%
%$ roughQRS (rough estimate of Q and S sample numbers)                $%
%$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$%
%$%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%$%
function [Qbs, Rbs, Sbs, Tbs] = findFiducialPointsSingleBeat(data,sampling_rate,run_type,threshold_value,roughQRS)
try
  %% Processing inputs
  % make run_type case-insensitive
  run_type = upper(run_type);
  
  % make sure data is right dimension
  data_dim = size(data);
  if data_dim(1,1) ~= 1
    data = data';
  end
  
  %% initialize figures for debugging mode
  old_figure_exist = 0;
  if strcmp(run_type,'DEBUG-A') || strcmp(run_type,'DEBUG-B')
    if ~isempty(findall(0,'Type','Figure'))
      old_figure = gcf;
      old_figure_exist = 1;
    end
    new_figure = figure; hold on;
    title('Original Data with Fiducial Points','FontSize',14,'FontName','Gill Sans');
    xlabel('Sample Number','FontSize',14,'FontName','Gill Sans');
    ylabel('Voltage (mV)','FontSize',14,'FontName','Gill Sans');
    set(new_figure,'Name','Confirmation A');
    if strcmp(run_type,'DEBUG-B')
      new_figure_2 = figure; hold on;
      title('Curve Length Data with Q and S Markup','FontSize',14,'FontName','Gill Sans');
      xlabel('Sample Number','FontSize',14,'FontName','Gill Sans');
      ylabel('Curve Length Area','FontSize',14,'FontName','Gill Sans');
      set(new_figure_2,'Name','Confirmation B');
      new_figure_3 = figure; hold on;
      title('Zeroed QRS Data','FontSize',14,'FontName','Gill Sans');
      xlabel('Sample Number','FontSize',14,'FontName','Gill Sans');
      ylabel('Voltage (mV)','FontSize',14,'FontName','Gill Sans');
      set(new_figure_3,'Name','Confirmation C');
      new_figure_4 = figure; hold on;
      title('Zeroed QRS Curve Length Data with End of T Markup','FontSize',14,'FontName','Gill Sans');
      xlabel('Sample Number','FontSize',14,'FontName','Gill Sans');
      ylabel('Curve Length Area','FontSize',14,'FontName','Gill Sans');
      set(new_figure_4,'Name','Confirmation D');
      new_figure_5 = figure; hold on;
      title('Zoomed In Zeroed QRS Curve Length Data with End of T Markup','FontSize',14,'FontName','Gill Sans');
      xlabel('Sample Number','FontSize',14,'FontName','Gill Sans');
      ylabel('Curve Length Area','FontSize',14,'FontName','Gill Sans');
      set(new_figure_5,'Name','Confirmation E');
      new_figure_6 = figure; hold on;
      title('Rotated T-end detection','FontSize',14,'FontName','Gill Sans');
      xlabel('Sample Number','FontSize',14,'FontName','Gill Sans');
      ylabel('Rotated Curve Length Area','FontSize',14,'FontName','Gill Sans');
      set(new_figure_6,'Name','Confirmation F');
    end
  end
  
  %% initialize counters
  i = 1; % iterate through each sample of curve_length_data
  crossing_counter = 1; % iterate each time a point crosses the current_threshold_value
  min_index_counter = 1; % iterate each time a minimum value is found
  max_index_counter = 1; % iterate each time a maximum value is found
  fiducial_point_count = 1; % iterate for each beat (within each beat f_p_c should be the same for Qbs and Sbs etc.)
  
  %% compute curve length tranformed data
  length_data = lengthTransform(130,data,2048,'QRS');
  
  %% set initial QRS threshold
  threshold_base_value = mean(length_data(roughQRS(1):roughQRS(2)));
  current_threshold = threshold_value*threshold_base_value; 
  
  %% Q and S decision algorithm
  % no adaptive thresholding was done as all of our data is only one beat
  while i < length(length_data)
    % current_threshold(i) = threshold_value*threshold_base_value; 
    if length_data(i) > current_threshold
      crossing_point(crossing_counter) = i;
      crossing_counter = crossing_counter + 1;
      
      % scan backwards 125 ms or if not possible as far as possible
      [L_min, index_min] = min(length_data(1,max(1,i-numSamp(125,sampling_rate,'ms')):i));
      min_indices(min_index_counter) = max(i-numSamp(125,sampling_rate,'ms'),1)+index_min;
      
      % scan forwards 125 ms or if not possible as far as possible
      [L_max, index_max] = max(length_data(1,i:min(length(length_data),i+numSamp(125,sampling_rate,'ms'))));
      max_indices(max_index_counter) = min(i+index_max,length(data));
      
      
      % compute LA and the threshold for Q and S points
      LA = L_max - L_min;
      Q_threshold = L_min + LA/100;
      S_threshold = L_max;
      
      % find Q_b by search backward until value < Q_threshold
      j = 1;
      while length_data(1,i-j) > Q_threshold
        if i - j <= 1
          break;
        end
        j = j + 1;
      end
      Qbs(fiducial_point_count) = i-j;
      
      % find S_b by search forward until value < S_threshold
      %%% need to fiddle with S_threshold for accurate S points %%%
      j = 1;
      while length_data(1,max_indices(max_index_counter) + j) > S_threshold
        if max_indices(max_index_counter)+j >= length(length_data)
          break;
        end
        j = j + 1;
      end
      Sbs(fiducial_point_count) = max_indices(max_index_counter)+j;
      
      % widen intervals by 10 samples
      Qbs(fiducial_point_count) = max(Qbs(fiducial_point_count) - 10,2);
      Sbs(fiducial_point_count) = min(Sbs(fiducial_point_count) + 10,length(length_data));
      
      % eye closing period for 250ms %%%% widened to 2000ms because we only have one beat and thus no need for realism%%%
%       for j = i:min(length(length_data),i+numSamp(2000,sampling_rate,'ms'));
%         current_threshold(j) = threshold_value*threshold_base_value;
%       end
      i = i + numSamp(2000,sampling_rate,'ms');
      
      fiducial_point_count = fiducial_point_count + 1;
      min_index_counter = min_index_counter + 1;
      max_index_counter = max_index_counter + 1;
    end
    i = i + 1;
  end
  
  %% R decisions algorithm
  for i = 1:length(Qbs)
    [max_R(i),max_R_index(i)] = max(data(Qbs(i):Sbs(i)));
    [min_R(i),min_R_index(i)] = min(data(Qbs(i):Sbs(i)));
    max_R_index(i) = max_R_index(i) + Qbs(i);
    min_R_index(i) = min_R_index(i) + Qbs(i);
    if abs(max_R(i)) > abs(min_R(i))
      Rbs(i) = max_R_index(i);
    else
      Rbs(i) = min_R_index(i);
    end
  end
  
  %% plot DEBUG-B data (curve length + markup) (ie FIGURE 2)
  if strcmp(run_type,'DEBUG-B')
    figure(new_figure_2); hold on;
    plot(length_data,'k');
    plot(crossing_point,length_data(crossing_point),'b*','MarkerSize',16);
    plot(min_indices,length_data(min_indices),'g*','MarkerSize',16);
    plot(max_indices,length_data(max_indices),'r*','MarkerSize',16);
    plot(Qbs,length_data(Qbs),'c*','MarkerSize',16);
    plot(Sbs,length_data(Sbs),'m*','MarkerSize',16);
    legend('Length Transform','Crossing Point (tc)','tc-125ms min','tc+125ms max','Q','S','Location','Best');
    plot(current_threshold);
    plot(Q_threshold,'r');
    set(gca,'FontSize',14,'FontName','Gill Sans');
  end
  
  %% zero QRS Complex
  zeroed_data = zeroQRScomplex(data,Qbs,Sbs);
  if strcmp(run_type,'DEBUG-B')
    figure(new_figure_3); hold on;
    plot(zeroed_data,'k');
    set(gca,'FontSize',14,'FontName','Gill Sans');
  end
  
  %% calculate length tranformed data from zeroed data
  zeroed_length_data = lengthTransform(200,zeroed_data,2048,'T');
  
  %% end of T search
  % find the indices necessary for the line
  iofstart = Sbs;
  iofend = length(zeroed_length_data);
  valueofend = zeroed_length_data(iofend);
  
  [valueofpeak, iofpeak] = max(zeroed_length_data(iofstart(i):iofend(i)));
  iofpeak = iofpeak + iofstart-1;
  
  % slope of the line from peak of T-wave to end of recording
  slope = (valueofend-valueofpeak)/(iofend-iofpeak);
  
  % angle to rotate about
  angle = -atan(slope);
  
  % create vectors to be rotated
  x1 = iofpeak:iofend;
  y1 = zeroed_length_data(iofpeak:iofend);
  v1 = [x1;y1]; % data vector
  
  x2 = [iofpeak iofend];
  y2 = [valueofpeak valueofend];
  v2 = [x2;y2]; % secant vector
  
  origin = [iofpeak; zeroed_length_data(iofpeak)]; % left most point
  z1 = repmat(origin,1,length(y1));
  z2 = repmat(origin,1,length(y2));
  
  % create rotation matrix
  R1 = [cos(angle) -sin(angle); sin(angle) cos(angle)];
  
  % rotate vectors
  v_rot_1 = R1*(v1 - z1) + z1; % rotated data
  v_rot_2 = R1*(v2 - z2) + z2; % rotated secant line
  
  % find minimum value of rotated vector
  [min_val, min_index] = min(v_rot_1(2,:));
  Tbs = min_index+iofpeak-1;
  
  %% plot DEBUG-A and part of DEBUG-B (data and detected fiducial points)
  if strcmp(run_type,'DEBUG-A') || strcmp(run_type,'DEBUG-B')
    figure(new_figure); hold on;
    plot(data,'k');
    plot(Qbs,data(Qbs),'b*','MarkerSize',16);
    plot(Rbs,data(Rbs),'g*','MarkerSize',16);
    plot(Sbs,data(Sbs),'r*','MarkerSize',16);
    plot(Tbs,data(Tbs),'c*','MarkerSize',16);
    legend('ECG','Q','R','S','T','Location','Best');
    set(gca,'FontSize',14,'FontName','Gill Sans');
  end
  
  %% plot DEBUG-B data (curve length + markup)
  if strcmp(run_type,'DEBUG-B')
    % plot zeroed curve length transfrom, and secant line with markup
    figure(new_figure_4); hold on;
    plot(zeroed_length_data,'k');
    line([iofpeak iofend], [valueofpeak valueofend]);
    plot(iofend,valueofend,'r*','MarkerSize',16);
    plot(iofpeak,zeroed_length_data(iofpeak),'g*','MarkerSize',16);
    plot(Tbs,zeroed_length_data(Tbs),'c*','MarkerSize',16);
    legend('Curve Length Data','Secant Line','End of Line','Start of Line','T','Location','Best');
    set(gca,'FontSize',14,'FontName','Gill Sans');
    
    % same as above just zoomed in
    figure(new_figure_5); hold on;
    axis([iofpeak-5 iofend+5 min(zeroed_length_data(iofpeak:iofend))-5 valueofpeak+5]);
    plot(zeroed_length_data,'k');
    line([iofpeak iofend], [valueofpeak valueofend]);
    plot(iofend,valueofend,'r*','MarkerSize',16);
    plot(iofpeak,zeroed_length_data(iofpeak),'g*','MarkerSize',16);
    plot(Tbs,zeroed_length_data(Tbs),'c*','MarkerSize',16);
    legend('Curve Length Data','Secant Line','End of Line','Start of Line','T','Location','Best');
    set(gca,'FontSize',14,'FontName','Gill Sans');
    
    % plot rotated data
    figure(new_figure_6); hold on;
    plot(v_rot_1(1,:),v_rot_1(2,:),'k');
    plot(v_rot_2(1,:),v_rot_2(2,:),'b');
    plot(v_rot_1(1,min_index),min_val,'r*','MarkerSize',16);
    legend('Rotated Curve Length Data','Rotated Secant Line','Minimum','Location','Best');
    set(gca,'FontSize',14,'FontName','Gill Sans');
  end
  
  %% reset current figure to what it was before fiducial point algorithm
  if strcmp(run_type,'DEBUG-A') || strcmp(run_type,'DEBUG-B')
    if old_figure_exist == 1
      figure(old_figure);
    end
  end
catch ME
  close all;
  fprintf(['Lowering Threshold to ', num2str(threshold_value-0.02),'\t ERROR MESSAGE: ', ME.message, '\n']);
  [Qbs, Rbs, Sbs, Tbs] = findFiducialPointsSingleBeat(data,sampling_rate,run_type,threshold_value-0.02,roughQRS);
end
end

